/*
 * Copyright Red Hat, Inc. and/or its affiliates
 * and other contributors as indicated by the @author tags and
 * the COPYRIGHT.txt file distributed with this work.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.teiid.dqp.internal.datamgr;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import org.teiid.adminapi.impl.VDBTranslatorMetaData;


public class TranslatorRepository implements Serializable {
	
	private static final long serialVersionUID = -1212280886010974273L;
	private Map<String, VDBTranslatorMetaData> translatorRepo = new ConcurrentHashMap<String, VDBTranslatorMetaData>();

	public void addTranslatorMetadata(String name, VDBTranslatorMetaData factory) {
		this.translatorRepo.put(name, factory);
	}	
	
	public VDBTranslatorMetaData getTranslatorMetaData(String name) {
		VDBTranslatorMetaData factory = this.translatorRepo.get(name);
		return factory;
	}
	
	public VDBTranslatorMetaData removeTranslatorMetadata(String name) {
		return this.translatorRepo.remove(name);
	}	
	
	public List<VDBTranslatorMetaData> getTranslators(){
		return new ArrayList<VDBTranslatorMetaData>(translatorRepo.values());
	}
}